﻿using System;
using System.IO;
using System.Net;
using System.Net.Sockets;
using System.Text;

namespace MessageTester
{
    class MessageTest
    {
        public static void Main()
        {
            string[] prefixes = { "http://192.168.1.104:13000/", "http://192.168.1.104:13001/" };
            HttpListenerTest(prefixes);
        }

        // This example requires the System and System.Net namespaces. 
        public static void HttpListenerTest(string[] prefixes)
        {
            if (!HttpListener.IsSupported)
            {
                Console.WriteLine("Windows XP SP2 or Server 2003 is required to use the HttpListener class.");
                return;
            }
            // URI prefixes are required, 
            // for example "http://contoso.com:8080/index/".
            if (prefixes == null || prefixes.Length == 0)
                throw new ArgumentException("prefixes");

            // Create a listener.
            HttpListener listener = new HttpListener();
            // Add the prefixes. 
            foreach (string s in prefixes)
            {
                listener.Prefixes.Add(s);
            }
            listener.Start();
            while (true)
            {
                Console.Write("Listening...");
                // Note: The GetContext method blocks while waiting for a request. 
                HttpListenerContext context = listener.GetContext();
                HttpListenerRequest request = context.Request;
                Console.WriteLine("Connection from {0}", request.UserHostAddress);
                // Obtain a response object.
                HttpListenerResponse response = context.Response;
                

                // Construct a response. 
                string responseString = "<HTML><BODY> Hello world!</BODY></HTML>";
                byte[] buffer = System.Text.Encoding.UTF8.GetBytes(responseString);

                // Get a response stream and write the response to it.
                response.ContentLength64 = buffer.Length;
                System.IO.Stream output = response.OutputStream;
                output.Write(buffer, 0, buffer.Length);
                // You must close the output stream.
                output.Close();
            }
            listener.Stop();
        }
 
        private static void  SocketListener()
        {
        TcpListener server = null;
            try
            {
                // Set the TcpListener to the computer's IP and port 13000.
                Int32 port = 13000;
                IPAddress localAddr = IPAddress.Parse("192.168.1.104");
                Console.WriteLine("Listener {0}:{1}",localAddr.ToString(), port.ToString());

                // TcpListener server = new TcpListener(port);
                server = new TcpListener(localAddr, port);

                // Start listening for client requests.
                server.Start();

                // Buffer for reading data
                Byte[] bytes = new Byte[256];
                String data = null;

                // Enter the listening loop. 
                while (true)
                {
                    Console.Write("Waiting for a connection... ");

                    // Perform a blocking call to accept requests. 
                    // You could also user server.AcceptSocket() here.
                    TcpClient client = server.AcceptTcpClient();
                    Console.WriteLine("Connected!");

                    data = null;

                    // Get a stream object for reading and writing
                    NetworkStream stream = client.GetStream();

                    int i;

                    // Loop to receive all the data sent by the client. 
                    while ((i = stream.Read(bytes, 0, bytes.Length)) > 1)
                    {
                        // Translate data bytes to a ASCII string.
                        data = System.Text.Encoding.ASCII.GetString(bytes, 0, i);
                        Console.WriteLine("Received[{0}]: \n{1}", i, data);

                        if(string.IsNullOrEmpty(data))
                            break;

                        // Process the data sent by the client.
                        //data = data.ToUpper();
                    }

                    byte[] msg = System.Text.Encoding.ASCII.GetBytes("HTTP/1.1 200 OK");

                    // Send a response.
                    stream.Write(msg, 0, msg.Length);
                    Console.WriteLine("Sent: \n{0}", msg);

                    // Shutdown and end connection
                    client.Close();
                }
            }
            catch (SocketException e)
            {
                Console.WriteLine("SocketException: {0}", e);
            }
            finally
            {
                // Stop listening for new clients.
                server.Stop();
            }


            Console.WriteLine("\nHit enter to continue...");
            Console.Read();
        }
    }
}
