﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Net;

/*
<Message xmlns:i="http://www.w3.org/2001/XMLSchema-instance" xmlns="http://schemas.datacontract.org/2004/07/AgaveRobotics.Web.Demos.Model>
    <Content>Hi W5200 from Spinneret.</Content>
    <From>spinneret</From>
    <Id>1</Id>
    <To>W5200</To>
</Message>
 */
namespace AgaveRobotics.Web.Demos.Model
{
    public class Message
    {
        public int Id { get; set; }
        public string From { get; set; }
        public string To { get; set; }
        public string Content { get; set; }
    }

    public interface IMessageRepository
    {
        Message Get(int id);
        IEnumerable<Message> GetAll();
        Message Add(Message item);
        void Remove(int id);
        void RemoveUsername(string username);
    }

    public class MessageRepository : IMessageRepository
    {
        //TODO: Implement queued messages when users are not connected
        private List<Message> messages = new List<Message>();
        private int identity = 1;
        

        public MessageRepository()
        {}

        public Message Get(int id)
        {
            return messages.Find(m => m.Id == id);
        }

        public IEnumerable<Message> GetAll()
        {
            return messages;
        }

        public Message Add(Message item)
        {
            if (item == null)
            {
                throw new ArgumentNullException("subscriber item");
            }
            item.Id = identity++;
            messages.Add(item);

            Send(item);

            //Remove(item.Id);

            return item;
        }

        public void Remove(int id)
        {
            messages.RemoveAll(m => m.Id == id);
        }

        public void RemoveUsername(string username)
        {
            messages.RemoveAll(m => m.From == username);
        }

        private void Send(Message item)
        {
            // Get the singleton subscription list
            List<Subscriber> subscribers = SubscriberSingleton.Instance.Subscribers;

            string status = string.Empty;
            //TODO: Handle disconnected mesages
            //We're assuming only one message record per TO: during testing.
            Subscriber subscriber = subscribers.Find(s => s.Username == item.To);

            if (subscriber != null)
            {
                status = HttpPostRequest(item, subscriber);

                //TODO: Remove message
            }

            //TODO: Handle subscriber HttpPostRequests response status and return bool

        }

        private string HttpPostRequest(Message message, Subscriber subscriber)
        {
            Stream dataStream = null;
            WebResponse response = null;
            string status = string.Empty;
            try
            {
                WebRequest request = WebRequest.Create(string.Format("http://{0}:{1}", subscriber.Ip, subscriber.Port.ToString()));

                // Create POST and convert the message to a byte array.
                // Message format in the body of the HTTP POST [from=username+content=message]
                request.Method = "POST";
                string note = string.Format("from={0}+content={1}\n", message.From, message.Content);
                byte[] byteArray = Encoding.UTF8.GetBytes(note);
                request.ContentType = "application/x-www-form-urlencoded";
                request.ContentLength = byteArray.Length;

                //  Write the data to the request stream.
                dataStream = request.GetRequestStream();
                dataStream.Write(byteArray, 0, byteArray.Length);
                dataStream.Close();

                // Get the response.
                response = request.GetResponse();

                // Get the status
                status = ((HttpWebResponse)response).StatusDescription;
                response.Close();
            }
            catch (Exception ex)
            {
                status = ex.Message;
            }
            return status;


        }

    }
}
