﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

/*
<Subscriber xmlns:i="http://www.w3.org/2001/XMLSchema-instance" xmlns="http://schemas.datacontract.org/2004/07/AgaveRobotics.Web.Demos.Model">
    <Id>1</Id>
    <Ip>192.168.1.104</Ip>
    <Port>13000</Port>
    <Username>W5200</Username>
</Subscriber>
 */
namespace AgaveRobotics.Web.Demos.Model
{

    public class Subscriber 
    {
        public int Id { get; set; }
        public string Username { get; set; }
        public string Ip { get; set; }
        public int Port { get; set; }
    }

    public interface ISubscriberRepository
    {
        IEnumerable<Subscriber> GetAll();
        Subscriber Get(int id);
        Subscriber Find(string username);
        Subscriber Add(Subscriber item);
        void Remove(int id);
        void RemoveUsername(string username);
        bool Update(Subscriber item);
    }


    public sealed class SubscriberSingleton
    {
        public List<Subscriber> Subscribers { get; set; }

        static readonly SubscriberSingleton instance = new SubscriberSingleton();
        public static SubscriberSingleton Instance
        {
            get { return instance; }
        }

        private SubscriberSingleton()
        {
            Subscribers = new List<Subscriber>();        
        }


    }

    public class SubscriberRepository : ISubscriberRepository
    {
        //private List<Subscriber> subscribers = new List<Subscriber>();
        private List<Subscriber> subscribers = SubscriberSingleton.Instance.Subscribers;
        private int identity = 1;

        public SubscriberRepository()
        { }

        public IEnumerable<Subscriber> GetAll()
        {
            return subscribers;
        }

        public Subscriber Get(int id)
        {
            return subscribers.Find(s => s.Id == id);
        }

        public Subscriber Find(string username)
        {
            return subscribers.Find(s => s.Username == username);
        }

        public Subscriber Add(Subscriber item)
        {
            if (item == null)
            {
                throw new ArgumentNullException("subscriber item");
            }

            if (Find(item.Username) == null)
            {
                item.Id = identity++;
                subscribers.Add(item);
                return item;
            }
            else {
                return item;
            }
        }

        public void Remove(int id)
        {
            subscribers.RemoveAll(s => s.Id == id);
        }

        public void RemoveUsername(string username)
        {
            subscribers.RemoveAll(s => s.Username == username);
        }

        public bool Update(Subscriber item)
        {
            if (item == null)
            {
                throw new ArgumentNullException("subscriber item");
            }
            int index = subscribers.FindIndex(p => p.Id == item.Id);
            if (index == -1)
            {
                return false;
            }
            subscribers.RemoveAt(index);
            subscribers.Add(item);
            return true;
        }
    }
}
